'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = initAppManager;

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _reconcile = require('./reconcile');

var _reconcile2 = _interopRequireDefault(_reconcile);

var _script = require('./script');

var _script2 = _interopRequireDefault(_script);

var _element = require('./element');

var _element2 = _interopRequireDefault(_element);

var _lifecycle = require('./lifecycle');

var _lifecycle2 = _interopRequireDefault(_lifecycle);

var _update = require('./update');

var _update2 = _interopRequireDefault(_update);

var _fragments = require('./fragments');

var _fragments2 = _interopRequireDefault(_fragments);

var _context = require('./utils/context');

var _context2 = _interopRequireDefault(_context);

var _constants = require('./constants');

var _config = require('./utils/config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initAppManager(w) {
  function appManager(config, overrideOptions) {
    var slots = (0, _config.setNames)(config.slots);
    var fragments = (0, _config.setNames)(config.fragments);
    var routes = (0, _config.setNames)(config.routes);

    var options = {
      importTimeout: _constants.IMPORT_TIMEOUT,
      getRouteNameFromResource: (0, _config.defaultGetRouteNameFromResource)(routes),
      getElement: (0, _config.initDefaultGetElement)(w),
      getAdditionalState: undefined
    };

    if (overrideOptions) {
      if (typeof overrideOptions.importTimeout === 'number') {
        (0, _assign2.default)(options, { importTimeout: overrideOptions.importTimeout });
      }

      if (typeof overrideOptions.getRouteNameFromResource === 'function') {
        (0, _assign2.default)(options, { getRouteNameFromResource: overrideOptions.getRouteNameFromResource });
      }

      if (typeof overrideOptions.getElement === 'function') {
        (0, _assign2.default)(options, { getElement: overrideOptions.getElement });
      }

      if (typeof overrideOptions.getAdditionalState === 'function') {
        (0, _assign2.default)(options, { getAdditionalState: overrideOptions.getAdditionalState });
      }
    }

    var getRouteNameFromResource = options.getRouteNameFromResource,
        getAdditionalState = options.getAdditionalState,
        importTimeout = options.importTimeout,
        getElement = options.getElement;


    var context = new _context2.default();

    var updateState = (0, _state2.default)(routes, getRouteNameFromResource, getAdditionalState);
    var getFragmentScript = (0, _script2.default)(fragments, importTimeout, context);
    var getSlotElement = (0, _element2.default)(slots, getElement, importTimeout);
    var createLifecycle = (0, _lifecycle2.default)(slots, getFragmentScript, getSlotElement, context);
    var getPageFragments = (0, _fragments2.default)(slots, fragments, routes);
    var reconcile = (0, _reconcile2.default)(getPageFragments);
    var updatePage = (0, _update2.default)(reconcile, createLifecycle, updateState, context);

    // $FlowFixMe until https://github.com/facebook/flow/issues/285 is fixed.
    Object.defineProperty(updatePage, 'state', {
      enumerable: true,
      get: function get() {
        return context.state;
      }
    });

    return updatePage;
  }

  appManager.statuses = _constants.statuses;
  appManager.levels = _constants.levels;

  return appManager;
}