'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

exports.default = initGetPageFragments;

var _slotFinder = require('slot-finder');

var _slotFinder2 = _interopRequireDefault(_slotFinder);

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initGetPageFragments(slots, fragments, routes) {
  return function getPageFragments(routeName) {
    var errData = {
      source: 'get_page_fragments',
      fragment: null,
      slot: null
    };

    var route = routes[routeName];

    if (!route) {
      throw new _appManagerError2.default('Route ' + routeName + ' is not specified in config.', (0, _extends4.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'missing_route',
        recoverable: false
      }));
    }

    var routeFragmentNames = [];
    if (Array.isArray(route.fragments)) {
      routeFragmentNames = route.fragments;
    } else if (typeof route.fragment === 'string') {
      routeFragmentNames = [route.fragment];
    }

    var routeFragments = routeFragmentNames.map(function (fragmentName) {
      return fragments[fragmentName];
    });

    var missingFragments = routeFragments.reduce(function (missing, fragment, index) {
      return fragment ? missing : missing.concat(routeFragmentNames[index]);
    }, []);
    if (missingFragments.length > 0) {
      throw new _appManagerError2.default('Route ' + routeName + ' tried to mount at least one missing fragment: ' + missingFragments.join(', '), (0, _extends4.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'missing_fragment',
        recoverable: false
      }));
    }

    var fragmentsWithSlots = routeFragments.map(function (fragment) {
      var fragmentSlots = [];
      if (Array.isArray(fragment.slots)) {
        fragmentSlots = fragment.slots;
      } else if (typeof fragment.slot === 'string') {
        fragmentSlots = [fragment.slot];
      }

      return { name: fragment.name, slots: fragmentSlots };
    });

    var emptySlots = fragmentsWithSlots.reduce(function (fragmentSlots, fragment) {
      return fragmentSlots.concat(fragment.slots);
    }, []).reduce(function (empty, slotName) {
      return (0, _extends4.default)({}, empty, (0, _defineProperty3.default)({}, slotName, null));
    }, {});

    var missingSlots = (0, _keys2.default)(emptySlots).reduce(function (missing, slotName) {
      return slots[slotName] ? missing : missing.concat(slotName);
    }, []);
    if (missingSlots.length > 0) {
      throw new _appManagerError2.default('Route ' + routeName + ' tried to mount into at least one missing slot: ' + missingSlots.join(', '), (0, _extends4.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'invalid_slots',
        recoverable: false
      }));
    }

    var pageFragments = void 0;

    try {
      pageFragments = (0, _slotFinder2.default)(emptySlots, fragmentsWithSlots);
    } catch (err) {
      throw new _appManagerError2.default(err, (0, _extends4.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'slot_finder',
        recoverable: true
      }));
    }

    if (!pageFragments || (typeof pageFragments === 'undefined' ? 'undefined' : (0, _typeof3.default)(pageFragments)) !== 'object') {
      throw new _appManagerError2.default('SlotFinder returned invalid fragments for route ' + routeName, (0, _extends4.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'get_fragments',
        recoverable: true
      }));
    }

    return pageFragments;
  };
}