'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends6 = require('babel-runtime/helpers/extends');

var _extends7 = _interopRequireDefault(_extends6);

exports.default = initReconcile;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initReconcile(getPageFragments) {
  return function reconcile(newRouteName, oldRouteName) {
    var errData = {
      slot: null,
      fragment: null,
      source: 'reconcile'
    };

    if (!newRouteName) {
      throw new _appManagerError2.default('No route set when trying to reconcile page.', (0, _extends7.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'no_current_route',
        recoverable: true
      }));
    }

    var newRouteFragments = getPageFragments(newRouteName);

    var newRouteSlots = (0, _keys2.default)(newRouteFragments);

    var mount = {};
    var update = {};
    var unmount = {};

    if (oldRouteName) {
      var oldRouteFragments = getPageFragments(oldRouteName);
      var oldRouteSlots = (0, _keys2.default)(oldRouteFragments);

      update = oldRouteSlots.filter(function (slotName) {
        var fragmentName = oldRouteFragments[slotName];
        return fragmentName && newRouteSlots.indexOf(slotName) > -1 && newRouteFragments[slotName] === fragmentName;
      }).reduce(function (diff, slotName) {
        return (0, _extends7.default)({}, diff, (0, _defineProperty3.default)({}, slotName, newRouteFragments[slotName]));
      }, {});

      unmount = oldRouteSlots.filter(function (slotName) {
        var fragmentName = oldRouteFragments[slotName];
        return fragmentName && (newRouteSlots.indexOf(slotName) === -1 || newRouteFragments[slotName] !== fragmentName);
      }).reduce(function (diff, slotName) {
        return (0, _extends7.default)({}, diff, (0, _defineProperty3.default)({}, slotName, oldRouteFragments[slotName]));
      }, {});

      mount = newRouteSlots.filter(function (slotName) {
        var fragmentName = newRouteFragments[slotName];
        return fragmentName && (oldRouteSlots.indexOf(slotName) === -1 || oldRouteFragments[slotName] !== fragmentName);
      }).reduce(function (diff, slotName) {
        return (0, _extends7.default)({}, diff, (0, _defineProperty3.default)({}, slotName, newRouteFragments[slotName]));
      }, {});
    } else {
      mount = newRouteSlots.filter(function (slotName) {
        return newRouteFragments[slotName];
      }).reduce(function (diff, slotName) {
        return (0, _extends7.default)({}, diff, (0, _defineProperty3.default)({}, slotName, newRouteFragments[slotName]));
      }, {});
    }

    return { unmount: unmount, mount: mount, update: update };
  };
}