'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.default = appManager;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _config = require('./utils/config');

var _constants = require('./constants');

var _reconcile2 = require('./reconcile');

var _reconcile3 = _interopRequireDefault(_reconcile2);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _fragments = require('./fragments');

var _fragments2 = _interopRequireDefault(_fragments);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function appManager(config) {
  var overrideOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var slots = (0, _config.setNames)(config.slots);
  var routes = (0, _config.setNames)(config.routes);
  var fragments = (0, _config.setNames)(config.fragments);
  var getPageFragments = (0, _fragments2.default)(slots, fragments, routes);
  var reconcile = (0, _reconcile3.default)(getPageFragments);

  var getRouteNameFromResource = overrideOptions.getRouteNameFromResource || (0, _config.defaultGetRouteNameFromResource)(routes);
  var updateState = (0, _state2.default)(routes, getRouteNameFromResource, overrideOptions.getAdditionalState);

  return {
    getState: updateState,

    getSlotsMarkup: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(resource) {
        var _this = this;

        for (var _len = arguments.length, getMarkupArgs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          getMarkupArgs[_key - 1] = arguments[_key];
        }

        var errData, state, newRouteName, _reconcile, mount, getMarkupPromises, resolvedMarkup;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                errData = {
                  source: 'get_slots_markup',
                  fragment: null,
                  slot: null
                };
                _context2.next = 3;
                return updateState({ resource: resource });

              case 3:
                state = _context2.sent;

                if (state) {
                  _context2.next = 6;
                  break;
                }

                throw new _appManagerError2.default('Could not find a route to match path', (0, _extends4.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'invalid_route',
                  recoverable: false
                }));

              case 6:
                newRouteName = state.route.name;
                _reconcile = reconcile(newRouteName), mount = _reconcile.mount;

                getMarkupPromises = function getMarkupPromises() {
                  return (0, _keys2.default)(mount).map(function () {
                    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slotName) {
                      var markup, fragmentName, fragment;
                      return _regenerator2.default.wrap(function _callee$(_context) {
                        while (1) {
                          switch (_context.prev = _context.next) {
                            case 0:
                              markup = '';
                              fragmentName = mount[slotName];

                              if (!fragmentName) {
                                _context.next = 8;
                                break;
                              }

                              fragment = fragments[fragmentName];

                              if (!(typeof fragment.getMarkup === 'function')) {
                                _context.next = 8;
                                break;
                              }

                              _context.next = 7;
                              return fragment.getMarkup.apply(fragment, [state].concat((0, _toConsumableArray3.default)(getMarkupArgs)));

                            case 7:
                              markup = _context.sent;

                            case 8:
                              return _context.abrupt('return', { slotName: slotName, markup: markup });

                            case 9:
                            case 'end':
                              return _context.stop();
                          }
                        }
                      }, _callee, _this);
                    }));

                    return function (_x3) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                };

                resolvedMarkup = void 0;
                _context2.prev = 10;
                _context2.next = 13;
                return _promise2.default.all(getMarkupPromises());

              case 13:
                resolvedMarkup = _context2.sent;
                _context2.next = 19;
                break;

              case 16:
                _context2.prev = 16;
                _context2.t0 = _context2['catch'](10);
                throw new _appManagerError2.default(_context2.t0, (0, _extends4.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'get_markup',
                  recoverable: false
                }));

              case 19:
                return _context2.abrupt('return', resolvedMarkup.reduce(function (acc, slotMarkup) {
                  return (0, _extends4.default)({}, acc, (0, _defineProperty3.default)({}, slotMarkup.slotName, slotMarkup.markup));
                }, {}));

              case 20:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[10, 16]]);
      }));

      function getSlotsMarkup(_x2) {
        return _ref.apply(this, arguments);
      }

      return getSlotsMarkup;
    }()
  };
}