'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.retry = exports.delay = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var delay = exports.delay = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(timeoutMs) {
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!(typeof timeoutMs !== 'number')) {
              _context.next = 2;
              break;
            }

            throw new _appManagerError2.default('Invalid config: importTimeout is not a number.', {
              level: _constants.levels.ERROR,
              code: 'invalid_timeout',
              recoverable: false,
              source: 'delay',
              fragment: null,
              slot: null
            });

          case 2:
            return _context.abrupt('return', new _promise2.default(function (resolve) {
              return setTimeout(resolve, timeoutMs);
            }));

          case 3:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function delay(_x) {
    return _ref.apply(this, arguments);
  };
}();

var retry = exports.retry = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(iteratee, interval) {
    var firstTry;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            firstTry = iteratee();

            if (!firstTry) {
              _context2.next = 3;
              break;
            }

            return _context2.abrupt('return', firstTry);

          case 3:
            return _context2.abrupt('return', new _promise2.default(function (resolve) {
              var iterator = setInterval(function () {
                var iterateeReturn = iteratee();

                if (iterateeReturn) {
                  resolve(iterateeReturn);
                  clearInterval(iterator);
                }
              }, interval);
            }));

          case 4:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function retry(_x2, _x3) {
    return _ref2.apply(this, arguments);
  };
}();

exports.setNames = setNames;
exports.defaultGetRouteNameFromResource = defaultGetRouteNameFromResource;
exports.initDefaultGetElement = initDefaultGetElement;

var _constants = require('../constants');

var _appManagerError = require('./app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function setNames(map) {
  var errData = {
    source: 'set_names',
    fragment: null,
    slot: null
  };

  if ((typeof map === 'undefined' ? 'undefined' : (0, _typeof3.default)(map)) !== 'object') {
    throw new _appManagerError2.default('Invalid config: map is not an object.', (0, _extends4.default)({}, errData, {
      level: _constants.levels.ERROR,
      code: 'invalid_map',
      recoverable: false
    }));
  }

  return (0, _keys2.default)(map).reduce(function (acc, key) {
    if ((0, _typeof3.default)(map[key]) !== 'object') {
      throw new _appManagerError2.default('Invalid config for ' + key + ': not an object.', (0, _extends4.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'invalid_map_item',
        recoverable: false
      }));
    }

    return (0, _extends4.default)({}, acc, (0, _defineProperty3.default)({}, key, (0, _extends4.default)({}, map[key], { name: key })));
  }, {});
}

function defaultGetRouteNameFromResource(routes) {
  if ((typeof routes === 'undefined' ? 'undefined' : (0, _typeof3.default)(routes)) !== 'object') {
    throw new _appManagerError2.default('Invalid config: routes is not an object.', {
      level: _constants.levels.ERROR,
      code: 'invalid_routes',
      recoverable: false,
      source: 'default_get_route_name_from_resource',
      fragment: null,
      slot: null
    });
  }

  return function getRouteNameFromResource(resource) {
    var _resource$split = resource.split('?'),
        _resource$split2 = (0, _slicedToArray3.default)(_resource$split, 1),
        path = _resource$split2[0];

    if (!path) {
      return null;
    }

    return (0, _keys2.default)(routes).find(function (routeName) {
      var route = routes[routeName];
      return Array.isArray(route.paths) ? route.paths.indexOf(path) !== -1 : path === route.path;
    });
  };
}

function initDefaultGetElement(w) {
  return function defaultGetElement(querySelector) {
    if (typeof querySelector !== 'string') {
      throw new _appManagerError2.default('Invalid config: bad querySelector passed to getElement,', {
        level: _constants.levels.ERROR,
        code: 'invalid_query_selector',
        recoverable: false,
        source: 'get_element',
        fragment: null,
        slot: null
      });
    }

    return retry(function () {
      return w.document.querySelector(querySelector);
    }, _constants.DOM_CHECK_INTERVAL);
  };
}